@@ -3,11 +3,11 @@
 use super::TreeOptions;
 use crate::core::compiler::{CompileKind, RustcTargetData};
 use crate::core::dependency::DepKind;
-use crate::core::resolver::features::{CliFeatures, FeaturesFor, ResolvedFeatures};
 use crate::core::resolver::Resolve;
+use crate::core::resolver::features::{CliFeatures, FeaturesFor, ResolvedFeatures};
 use crate::core::{FeatureMap, FeatureValue, Package, PackageId, PackageIdSpec, Workspace};
-use crate::util::interning::{InternedString, INTERNED_DEFAULT};
 use crate::util::CargoResult;
+use crate::util::interning::{INTERNED_DEFAULT, InternedString};
 use std::collections::{HashMap, HashSet};
 
 #[derive(Debug, Copy, Clone)]
@@ -453,10 +453,14 @@ fn add_pkg(
                 if !opts.edge_kinds.contains(&EdgeKind::Dep(dep.kind())) {
                     return false;
                 }
-                // Filter out proc-macrcos if requested.
+                // Filter out proc-macros if requested.
                 if opts.no_proc_macro && graph.package_for_id(dep_id).proc_macro() {
                     return false;
                 }
+                // Filter out private dependencies if requested.
+                if opts.public && !dep.is_public() {
+                    return false;
+                }
                 if dep.is_optional() {
                     // If the new feature resolver does not enable this
                     // optional dep, then don't use it.
@@ -546,10 +550,12 @@ fn add_pkg(
         }
     }
     if opts.graph_features {
-        assert!(graph
-            .dep_name_map
-            .insert(from_index, dep_name_map)
-            .is_none());
+        assert!(
+            graph
+                .dep_name_map
+                .insert(from_index, dep_name_map)
+                .is_none()
+        );
     }
 
     from_index
