@@ -120,10 +120,85 @@ impl<'a> Object<'a> {
         name
     }
 
+    pub(crate) fn elf_section_flags(&self, section: &Section<'_>) -> SectionFlags {
+        let sh_flags = match section.kind {
+            SectionKind::Text => elf::SHF_ALLOC | elf::SHF_EXECINSTR,
+            SectionKind::Data | SectionKind::ReadOnlyDataWithRel => elf::SHF_ALLOC | elf::SHF_WRITE,
+            SectionKind::Tls => elf::SHF_ALLOC | elf::SHF_WRITE | elf::SHF_TLS,
+            SectionKind::UninitializedData => elf::SHF_ALLOC | elf::SHF_WRITE,
+            SectionKind::UninitializedTls => elf::SHF_ALLOC | elf::SHF_WRITE | elf::SHF_TLS,
+            SectionKind::ReadOnlyData => elf::SHF_ALLOC,
+            SectionKind::ReadOnlyString => elf::SHF_ALLOC | elf::SHF_STRINGS | elf::SHF_MERGE,
+            SectionKind::OtherString | SectionKind::DebugString => {
+                elf::SHF_STRINGS | elf::SHF_MERGE
+            }
+            SectionKind::Other
+            | SectionKind::Debug
+            | SectionKind::Metadata
+            | SectionKind::Linker
+            | SectionKind::Note
+            | SectionKind::Elf(_) => 0,
+            SectionKind::Unknown | SectionKind::Common | SectionKind::TlsVariables => {
+                return SectionFlags::None;
+            }
+        }
+        .into();
+        SectionFlags::Elf { sh_flags }
+    }
+
+    pub(crate) fn elf_symbol_flags(&self, symbol: &Symbol) -> SymbolFlags<SectionId, SymbolId> {
+        let st_type = match symbol.kind {
+            SymbolKind::Text => {
+                if symbol.is_undefined() {
+                    elf::STT_NOTYPE
+                } else {
+                    elf::STT_FUNC
+                }
+            }
+            SymbolKind::Data => {
+                if symbol.is_undefined() {
+                    elf::STT_NOTYPE
+                } else if symbol.is_common() {
+                    elf::STT_COMMON
+                } else {
+                    elf::STT_OBJECT
+                }
+            }
+            SymbolKind::Section => elf::STT_SECTION,
+            SymbolKind::File => elf::STT_FILE,
+            SymbolKind::Tls => elf::STT_TLS,
+            SymbolKind::Label => elf::STT_NOTYPE,
+            SymbolKind::Unknown => {
+                if symbol.is_undefined() {
+                    elf::STT_NOTYPE
+                } else {
+                    return SymbolFlags::None;
+                }
+            }
+        };
+        let st_bind = if symbol.weak {
+            elf::STB_WEAK
+        } else if symbol.is_undefined() {
+            elf::STB_GLOBAL
+        } else if symbol.is_local() {
+            elf::STB_LOCAL
+        } else {
+            elf::STB_GLOBAL
+        };
+        let st_info = (st_bind << 4) + st_type;
+        let st_other = if symbol.scope == SymbolScope::Linkage {
+            elf::STV_HIDDEN
+        } else {
+            elf::STV_DEFAULT
+        };
+        SymbolFlags::Elf { st_info, st_other }
+    }
+
     fn elf_has_relocation_addend(&self) -> Result<bool> {
         Ok(match self.architecture {
             Architecture::Aarch64 => true,
             Architecture::Aarch64_Ilp32 => true,
+            Architecture::Alpha => true,
             Architecture::Arm => false,
             Architecture::Avr => true,
             Architecture::Bpf => false,
@@ -133,6 +208,7 @@ impl<'a> Object<'a> {
             Architecture::I386 => false,
             Architecture::X86_64 => true,
             Architecture::X86_64_X32 => true,
+            Architecture::Hppa => false,
             Architecture::Hexagon => true,
             Architecture::LoongArch32 => true,
             Architecture::LoongArch64 => true,
@@ -180,6 +256,7 @@ impl<'a> Object<'a> {
         let unsupported_reloc = || Err(Error(format!("unimplemented ELF relocation {:?}", reloc)));
         let r_type = match self.architecture {
             Architecture::Aarch64 => match (kind, encoding, size) {
+                (K::None, E::Generic, 0) => elf::R_AARCH64_NONE,
                 (K::Absolute, E::Generic, 64) => elf::R_AARCH64_ABS64,
                 (K::Absolute, E::Generic, 32) => elf::R_AARCH64_ABS32,
                 (K::Absolute, E::Generic, 16) => elf::R_AARCH64_ABS16,
@@ -191,29 +268,46 @@ impl<'a> Object<'a> {
                 _ => return unsupported_reloc(),
             },
             Architecture::Aarch64_Ilp32 => match (kind, encoding, size) {
+                (K::None, E::Generic, 0) => elf::R_AARCH64_NONE,
                 (K::Absolute, E::Generic, 32) => elf::R_AARCH64_P32_ABS32,
                 _ => return unsupported_reloc(),
             },
+            Architecture::Alpha => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_ALPHA_NONE,
+                // Absolute
+                (K::Absolute, _, 32) => elf::R_ALPHA_REFLONG,
+                (K::Absolute, _, 64) => elf::R_ALPHA_REFQUAD,
+                // Relative to the PC
+                (K::Relative, _, 16) => elf::R_ALPHA_SREL16,
+                (K::Relative, _, 32) => elf::R_ALPHA_SREL32,
+                (K::Relative, _, 64) => elf::R_ALPHA_SREL64,
+                _ => return unsupported_reloc(),
+            },
             Architecture::Arm => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_ARM_NONE,
                 (K::Absolute, _, 32) => elf::R_ARM_ABS32,
                 _ => return unsupported_reloc(),
             },
             Architecture::Avr => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_AVR_NONE,
                 (K::Absolute, _, 32) => elf::R_AVR_32,
                 (K::Absolute, _, 16) => elf::R_AVR_16,
                 _ => return unsupported_reloc(),
             },
             Architecture::Bpf => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_BPF_NONE,
                 (K::Absolute, _, 64) => elf::R_BPF_64_64,
                 (K::Absolute, _, 32) => elf::R_BPF_64_32,
                 _ => return unsupported_reloc(),
             },
             Architecture::Csky => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_CKCORE_NONE,
                 (K::Absolute, _, 32) => elf::R_CKCORE_ADDR32,
                 (K::Relative, E::Generic, 32) => elf::R_CKCORE_PCREL32,
                 _ => return unsupported_reloc(),
             },
             Architecture::I386 => match (kind, size) {
+                (K::None, 0) => elf::R_386_NONE,
                 (K::Absolute, 32) => elf::R_386_32,
                 (K::Relative, 32) => elf::R_386_PC32,
                 (K::Got, 32) => elf::R_386_GOT32,
@@ -227,6 +321,7 @@ impl<'a> Object<'a> {
                 _ => return unsupported_reloc(),
             },
             Architecture::E2K32 | Architecture::E2K64 => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_E2K_NONE,
                 (K::Absolute, E::Generic, 32) => elf::R_E2K_32_ABS,
                 (K::Absolute, E::E2KLit, 64) => elf::R_E2K_64_ABS_LIT,
                 (K::Absolute, E::Generic, 64) => elf::R_E2K_64_ABS,
@@ -235,6 +330,7 @@ impl<'a> Object<'a> {
                 _ => return unsupported_reloc(),
             },
             Architecture::X86_64 | Architecture::X86_64_X32 => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_X86_64_NONE,
                 (K::Absolute, E::Generic, 64) => elf::R_X86_64_64,
                 (K::Relative, E::X86Branch, 32) => elf::R_X86_64_PLT32,
                 (K::Relative, _, 32) => elf::R_X86_64_PC32,
@@ -249,11 +345,19 @@ impl<'a> Object<'a> {
                 (K::Relative, _, 8) => elf::R_X86_64_PC8,
                 _ => return unsupported_reloc(),
             },
+            Architecture::Hppa => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_PARISC_NONE,
+                (K::Absolute, _, 32) => elf::R_PARISC_DIR32,
+                (K::Relative, _, 32) => elf::R_PARISC_PCREL32,
+                _ => return unsupported_reloc(),
+            },
             Architecture::Hexagon => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_HEX_NONE,
                 (K::Absolute, _, 32) => elf::R_HEX_32,
                 _ => return unsupported_reloc(),
             },
             Architecture::LoongArch32 | Architecture::LoongArch64 => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_LARCH_NONE,
                 (K::Absolute, _, 32) => elf::R_LARCH_32,
                 (K::Absolute, _, 64) => elf::R_LARCH_64,
                 (K::Relative, _, 32) => elf::R_LARCH_32_PCREL,
@@ -267,6 +371,7 @@ impl<'a> Object<'a> {
                 _ => return unsupported_reloc(),
             },
             Architecture::M68k => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_68K_NONE,
                 (K::Absolute, _, 8) => elf::R_68K_8,
                 (K::Absolute, _, 16) => elf::R_68K_16,
                 (K::Absolute, _, 32) => elf::R_68K_32,
@@ -286,6 +391,7 @@ impl<'a> Object<'a> {
             },
             Architecture::Mips | Architecture::Mips64 | Architecture::Mips64_N32 => {
                 match (kind, encoding, size) {
+                    (K::None, _, 0) => elf::R_MIPS_NONE,
                     (K::Absolute, _, 16) => elf::R_MIPS_16,
                     (K::Absolute, _, 32) => elf::R_MIPS_32,
                     (K::Absolute, _, 64) => elf::R_MIPS_64,
@@ -293,26 +399,31 @@ impl<'a> Object<'a> {
                 }
             }
             Architecture::Msp430 => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_MSP430_NONE,
                 (K::Absolute, _, 32) => elf::R_MSP430_32,
                 (K::Absolute, _, 16) => elf::R_MSP430_16_BYTE,
                 _ => return unsupported_reloc(),
             },
             Architecture::PowerPc => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_PPC_NONE,
                 (K::Absolute, _, 32) => elf::R_PPC_ADDR32,
                 _ => return unsupported_reloc(),
             },
             Architecture::PowerPc64 => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_PPC64_NONE,
                 (K::Absolute, _, 32) => elf::R_PPC64_ADDR32,
                 (K::Absolute, _, 64) => elf::R_PPC64_ADDR64,
                 _ => return unsupported_reloc(),
             },
             Architecture::Riscv32 | Architecture::Riscv64 => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_RISCV_NONE,
                 (K::Absolute, _, 32) => elf::R_RISCV_32,
                 (K::Absolute, _, 64) => elf::R_RISCV_64,
                 (K::Relative, E::Generic, 32) => elf::R_RISCV_32_PCREL,
                 _ => return unsupported_reloc(),
             },
             Architecture::S390x => match (kind, encoding, size) {
+                (K::None, E::Generic, 0) => elf::R_390_NONE,
                 (K::Absolute, E::Generic, 8) => elf::R_390_8,
                 (K::Absolute, E::Generic, 16) => elf::R_390_16,
                 (K::Absolute, E::Generic, 32) => elf::R_390_32,
@@ -336,6 +447,7 @@ impl<'a> Object<'a> {
                 _ => return unsupported_reloc(),
             },
             Architecture::Sbf => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_SBF_NONE,
                 (K::Absolute, _, 64) => elf::R_SBF_64_64,
                 (K::Absolute, _, 32) => elf::R_SBF_64_32,
                 _ => return unsupported_reloc(),
@@ -356,22 +468,26 @@ impl<'a> Object<'a> {
                 _ => return unsupported_reloc(),
             },
             Architecture::Sparc | Architecture::Sparc32Plus => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_SPARC_NONE,
                 // TODO: use R_SPARC_32 if aligned.
                 (K::Absolute, _, 32) => elf::R_SPARC_UA32,
                 _ => return unsupported_reloc(),
             },
             Architecture::Sparc64 => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_SPARC_NONE,
                 // TODO: use R_SPARC_32/R_SPARC_64 if aligned.
                 (K::Absolute, _, 32) => elf::R_SPARC_UA32,
                 (K::Absolute, _, 64) => elf::R_SPARC_UA64,
                 _ => return unsupported_reloc(),
             },
             Architecture::SuperH => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_SH_NONE,
                 (K::Absolute, _, 32) => elf::R_SH_DIR32,
                 (K::Relative, _, 32) => elf::R_SH_REL32,
                 _ => return unsupported_reloc(),
             },
             Architecture::Xtensa => match (kind, encoding, size) {
+                (K::None, _, 0) => elf::R_XTENSA_NONE,
                 (K::Absolute, _, 32) => elf::R_XTENSA_32,
                 (K::Relative, E::Generic, 32) => elf::R_XTENSA_32_PCREL,
                 _ => return unsupported_reloc(),
@@ -563,6 +679,7 @@ impl<'a> Object<'a> {
         let e_machine = match (self.architecture, self.sub_architecture) {
             (Architecture::Aarch64, None) => elf::EM_AARCH64,
             (Architecture::Aarch64_Ilp32, None) => elf::EM_AARCH64,
+            (Architecture::Alpha, None) => elf::EM_ALPHA,
             (Architecture::Arm, None) => elf::EM_ARM,
             (Architecture::Avr, None) => elf::EM_AVR,
             (Architecture::Bpf, None) => elf::EM_BPF,
@@ -572,6 +689,7 @@ impl<'a> Object<'a> {
             (Architecture::I386, None) => elf::EM_386,
             (Architecture::X86_64, None) => elf::EM_X86_64,
             (Architecture::X86_64_X32, None) => elf::EM_X86_64,
+            (Architecture::Hppa, None) => elf::EM_PARISC,
             (Architecture::Hexagon, None) => elf::EM_HEXAGON,
             (Architecture::LoongArch32, None) => elf::EM_LOONGARCH,
             (Architecture::LoongArch64, None) => elf::EM_LOONGARCH,
@@ -639,59 +757,12 @@ impl<'a> Object<'a> {
         // Write symbols.
         writer.write_null_symbol();
         let mut write_symbol = |index: usize, symbol: &Symbol| -> Result<()> {
-            let st_info = if let SymbolFlags::Elf { st_info, .. } = symbol.flags {
-                st_info
-            } else {
-                let st_type = match symbol.kind {
-                    SymbolKind::Text => {
-                        if symbol.is_undefined() {
-                            elf::STT_NOTYPE
-                        } else {
-                            elf::STT_FUNC
-                        }
-                    }
-                    SymbolKind::Data => {
-                        if symbol.is_undefined() {
-                            elf::STT_NOTYPE
-                        } else if symbol.is_common() {
-                            elf::STT_COMMON
-                        } else {
-                            elf::STT_OBJECT
-                        }
-                    }
-                    SymbolKind::Section => elf::STT_SECTION,
-                    SymbolKind::File => elf::STT_FILE,
-                    SymbolKind::Tls => elf::STT_TLS,
-                    SymbolKind::Label => elf::STT_NOTYPE,
-                    SymbolKind::Unknown => {
-                        if symbol.is_undefined() {
-                            elf::STT_NOTYPE
-                        } else {
-                            return Err(Error(format!(
-                                "unimplemented symbol `{}` kind {:?}",
-                                symbol.name().unwrap_or(""),
-                                symbol.kind
-                            )));
-                        }
-                    }
-                };
-                let st_bind = if symbol.weak {
-                    elf::STB_WEAK
-                } else if symbol.is_undefined() {
-                    elf::STB_GLOBAL
-                } else if symbol.is_local() {
-                    elf::STB_LOCAL
-                } else {
-                    elf::STB_GLOBAL
-                };
-                (st_bind << 4) + st_type
-            };
-            let st_other = if let SymbolFlags::Elf { st_other, .. } = symbol.flags {
-                st_other
-            } else if symbol.scope == SymbolScope::Linkage {
-                elf::STV_HIDDEN
-            } else {
-                elf::STV_DEFAULT
+            let SymbolFlags::Elf { st_info, st_other } = self.symbol_flags(symbol) else {
+                return Err(Error(format!(
+                    "unimplemented symbol `{}` kind {:?}",
+                    symbol.name().unwrap_or(""),
+                    symbol.kind
+                )));
             };
             let (st_shndx, section) = match symbol.section {
                 SymbolSection::None => {
@@ -774,39 +845,12 @@ impl<'a> Object<'a> {
                 SectionKind::Elf(sh_type) => sh_type,
                 _ => elf::SHT_PROGBITS,
             };
-            let sh_flags = if let SectionFlags::Elf { sh_flags } = section.flags {
-                sh_flags
-            } else {
-                match section.kind {
-                    SectionKind::Text => elf::SHF_ALLOC | elf::SHF_EXECINSTR,
-                    SectionKind::Data | SectionKind::ReadOnlyDataWithRel => {
-                        elf::SHF_ALLOC | elf::SHF_WRITE
-                    }
-                    SectionKind::Tls => elf::SHF_ALLOC | elf::SHF_WRITE | elf::SHF_TLS,
-                    SectionKind::UninitializedData => elf::SHF_ALLOC | elf::SHF_WRITE,
-                    SectionKind::UninitializedTls => elf::SHF_ALLOC | elf::SHF_WRITE | elf::SHF_TLS,
-                    SectionKind::ReadOnlyData => elf::SHF_ALLOC,
-                    SectionKind::ReadOnlyString => {
-                        elf::SHF_ALLOC | elf::SHF_STRINGS | elf::SHF_MERGE
-                    }
-                    SectionKind::OtherString | SectionKind::DebugString => {
-                        elf::SHF_STRINGS | elf::SHF_MERGE
-                    }
-                    SectionKind::Other
-                    | SectionKind::Debug
-                    | SectionKind::Metadata
-                    | SectionKind::Linker
-                    | SectionKind::Note
-                    | SectionKind::Elf(_) => 0,
-                    SectionKind::Unknown | SectionKind::Common | SectionKind::TlsVariables => {
-                        return Err(Error(format!(
-                            "unimplemented section `{}` kind {:?}",
-                            section.name().unwrap_or(""),
-                            section.kind
-                        )));
-                    }
-                }
-                .into()
+            let SectionFlags::Elf { sh_flags } = self.section_flags(section) else {
+                return Err(Error(format!(
+                    "unimplemented section `{}` kind {:?}",
+                    section.name().unwrap_or(""),
+                    section.kind
+                )));
             };
             // TODO: not sure if this is correct, maybe user should determine this
             let sh_entsize = match section.kind {
