@@ -1,6 +1,6 @@
 // MIT License
 
-// Copyright (c) 2024-2025 The orion Developers
+// Copyright (c) 2024-2026 The orion Developers
 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -152,7 +152,7 @@ mod public {
     #[cfg(feature = "safe_api")]
     fn test_debug_impl() {
         let initial_state = Shake128::new();
-        let debug = format!("{:?}", initial_state);
+        let debug = format!("{initial_state:?}");
         let expected = "Shake128 { _state: State { state: [***OMITTED***], buffer: [***OMITTED***], capacity: 32, until_absorb: 0, to_squeeze: 0, is_finalized: false } }";
         assert_eq!(debug, expected);
     }
