@@ -1,6 +1,6 @@
 // MIT License
 
-// Copyright (c) 2018-2025 The orion Developers
+// Copyright (c) 2018-2026 The orion Developers
 // Based on the algorithm from https://github.com/floodyberry/poly1305-donna
 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -398,7 +398,7 @@ mod public {
     fn test_debug_impl() {
         let secret_key = OneTimeKey::generate();
         let initial_state = Poly1305::new(&secret_key);
-        let debug = format!("{:?}", initial_state);
+        let debug = format!("{initial_state:?}");
         let expected = "Poly1305 { a: [***OMITTED***], r: [***OMITTED***], s: [***OMITTED***], leftover: [***OMITTED***], buffer: [***OMITTED***], is_finalized: false }";
         assert_eq!(debug, expected);
     }
